#!/usr/bin/env bash
###
#File: start_remote_jupyter_server.sh
#Created: Thursday, 6th October 2022 6:21:43 pm
#Author: Jannek Squar (jannek.squar@uni-hamburg.de)
#-----
#
#-----
#Last Modified: Thursday, 6th October 2022 9:47:10 pm
#Modified By: Jannek Squar (jannek.squar@uni-hamburg.de)
#-----
#Copyright (c) 2022 Jannek Squar
#
###

: ${DKRZ_USER_ACCOUNT:=$1}
: ${DKRZ_GROUP_ACCOUNT:=$2}
: ${WORKSHOP_ENV:=python_workshop}

function usage {
	echo "Call script like this:"
	echo "./start_remote_jupyter_server.sh \$DKRZ_USER_ACCOUNT" \$DKRZ_WORKSHOP_ACCOUNT
	echo ""
	echo "\$DKRZ_USER_ACCOUNT: Your personal user or workshop account on Levante"
	echo "\$DKRZ_WORKSHOP_ACCOUNT: Your group account, via which you may submit SLURM jobs"
	exit 1
}

# Step 0: Check usage of script
if [[ "$#" -ne 2 ]]; then
	usage
fi

# Step 1: Setup levante environment:
	# 1. Check conda environment
	# 2. Create SLURM job script
	# 3. Submit SLURM job with Jupyter Lab server
	# 4. Wait until job is running

ssh -T $DKRZ_USER_ACCOUNT@levante.dkrz.de <<-EOL
	module load python3

	echo "+++++++++++++++++++++++++++++++++++++++++++"
	echo "Check conda environment"
	echo "+++++++++++++++++++++++++++++++++++++++++++"
	# setup conda environment
	if [[ ! \$(conda info --env | grep $WORKSHOP_ENV) ]]; then
		echo "Workshop environment $WORKSHOP_ENV has not been found and will now be created"
		conda create --yes --name $WORKSHOP_ENV
		echo "Install jupyterlab in environment $WORKSHOP_ENV"
		conda install --yes --name $WORKSHOP_ENV -c conda-forge jupyterlab
		conda init bash
	else
		echo "Workshop environment $WORKSHOP_ENV has been found"
	fi
	echo "Activate conda environment $WORKSHOP_ENV"
	source \$HOME/.bashrc
	conda activate $WORKSHOP_ENV

	# create slurm script
	DKRZ_GROUP_ACCOUNT=$DKRZ_GROUP_ACCOUNT
	echo "+++++++++++++++++++++++++++++++++++++++++++"
	echo "Submit SLURM job for Group Account \$DKRZ_GROUP_ACCOUNT"
	echo "+++++++++++++++++++++++++++++++++++++++++++"

	# check if job script already exists
	if [[ ! -f start_jupyter_server.sh ]]; then
		echo "Create job script start_jupyter_server.sh"
		cat <<-EOL_NESTED > start_jupyter_server.sh
			#!/bin/bash
			# launch jupyter

			#SBATCH --account=\$DKRZ_GROUP_ACCOUNT
			#SBATCH -J jupyter-server
			#SBATCH --time=8:00:00
			#SBATCH --mem=5G
			#SBATCH --partition interactive

			# Setup Environment
			source ~/.bashrc
			conda activate $WORKSHOP_ENV

			jupyter lab --no-browser --ip "*" --notebook-dir \$HOME
		EOL_NESTED
	else
		echo "Reuse existing job script start_jupyter_server.sh"
	fi

	# submit SLURM job
	JOB_ID=\$(sbatch start_jupyter_server.sh | cut -d " " -f 4)

	echo "+++++++++++++++++++++++++++++++++++++++++++"
	echo "Wait until job \$JOB_ID is running"
	echo "+++++++++++++++++++++++++++++++++++++++++++"

	while true; do
		JOB_QUEUE=\$(squeue |grep \$JOB_ID)
		NODE=\$(echo \$JOB_QUEUE | cut -d " " -f 8)
		STATE=\$(echo \$JOB_QUEUE | cut -d " " -f 5)
		sleep 1s
		if [[ -z \$STATE ]]; then
			echo "Queue: \$JOB_QUEUE"
			echo "State for job \$JOB_ID is unknown. Terminate script"
			exit 1
		fi
		if [[ \$STATE == R ]]; then
			echo ""
			echo "Job \$JOB_ID is running on node \$NODE"
			break
		else
			echo -n "."
		fi
	done
	echo ""
	echo "You need now to create tunnel to interactive node on levante if not already happened."
	echo "You can connect via your local webbrowser via the URL localhost:8888"
	echo "Do not forget to cancel your job on Levante after you are finished"	
	echo ""
	echo "+++++++++++++++++++++++++++++++++++++++++++"
	echo "Use the following command to create the tunnel:"
	echo "ssh -L 8888:localhost:8888 -J $DKRZ_USER_ACCOUNT@levante.dkrz.de $DKRZ_USER_ACCOUNT@\$NODE"
	echo "+++++++++++++++++++++++++++++++++++++++++++"
EOL


